/*******************************************************************************
* Project Name:      AN56377 - Project 3: Increasing USB Throughput with DMA 
* File Name:         main.c
* Version:           v2.0
*
* Devices Used:      CY8C3866AXI-040 & CY8C5868AXI-LP035
* Software Used:     PSoC Creator v3.0 SP1 (Build 3.0.0.3023)
* Compiler Used:     GNU-CC, MDK, RVDS
* Related Hardware:  CY8CKIT-001, CY8CKIT-030, CY8CKIT-050
********************************************************************************
* Theory of Operation:
*
* This example project is intended to demonstrate how use the DMA to move data 
* from the USB endpoint memory into a RAM location. Since the intention of this
* example is to increase overall throughput, the DMA along with an isochronous
* endpoint will be used. A similat concept would be used for interrupt and bulk 
* transfers.
*
********************************************************************************
* Copyright (2011-2014), Cypress Semiconductor Corporation.
********************************************************************************
* This software is owned by Cypress Semiconductor Corporation (Cypress)
* and is protected by and subject to worldwide patent protection (United
* States and foreign), United States copyright laws and international treaty
* provisions. Cypress hereby grants to licensee a personal, non-exclusive,
* non-transferable license to copy, use, modify, create derivative works of,
* and compile the Cypress Source Code and derivative works for the sole
* purpose of creating custom software in support of licensee product to be
* used only in conjunction with a Cypress integrated circuit as specified in
* the applicable agreement. Any reproduction, modification, translation,
* compilation, or representation of this software except as specified above
* is prohibited without the express written permission of Cypress.
*
* Disclaimer: CYPRESS MAKES NO WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, WITH
* REGARD TO THIS MATERIAL, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
* Cypress reserves the right to make changes without further notice to the
* materials described herein. Cypress does not assume any liability arising out
* of the application or use of any product or circuit described herein. Cypress
* does not authorize its products for use as critical components in life-support
* systems where a malfunction or failure may reasonably be expected to result in
* significant injury to the user. The inclusion of Cypress' product in a life-
* support systems application implies that the manufacturer assumes all risk of
* such use and in doing so indemnifies Cypress against all charges. Use may be
* limited by and subject to the applicable Cypress software license agreement.
*******************************************************************************/

#include <device.h>
#include <cyapicallbacks.h>

/* Create Defines for Endpoints */
#define EP1 0x01

/* Create defines for Unassigned Address and Interface Numbers */
#define ZeroBandwidthInterface 0
#define UnassignedAddress 0
#define BufferSize 64

extern volatile uint8 USBFS_deviceAddress;
uint8 AltSettingNumber = 0;

uint8 EP1_RAM[BufferSize];
uint8 OutBuffer[BufferSize]  ;/* Buffer to store the data received on the Out endpoint */
volatile uint8 InDataReady;
volatile uint8 OutFlag = 1;

void USBFS_EP_2_ISR_EntryCallback() {
	OutFlag = 0;
}

int main (void)
{
	/* Enable global interrupts */
	CYGlobalIntEnable;
	
	/* Start USBFS Component based on power settings in DWR (5V in this case) */
	USBFS_Start(0, USBFS_DWR_VDDD_OPERATION); 

	/* Pre-load the endpoint RAM buffer with data */
    //for(i=0;i<sizeof(EP1_RAM);i++) EP1_RAM[i] = i;	
	
    for(;;)
    {
        if (OutFlag == 0)
        {
			OutFlag = 1;
			for(uint8 i=0;i<sizeof(EP1_RAM);i++) EP1_RAM[i] = OutBuffer[i];
			USBFS_EnableOutEP(2);
			InDataReady = 1;
            
         }   
		/* Wait for Configuration to Complete */
		if(USBFS_GetConfiguration() != 0)
		{
			/* Reinitialize after SET_CONFIGURATION or SET_INTERFACE Requests */
			if(USBFS_IsConfigurationChanged() != ZeroBandwidthInterface)
			{
				/* Get the active interface number and reinitalize DMA */
				AltSettingNumber = USBFS_GetInterfaceSetting(0);
                
				/* Check that ISO Interface is active (Not Interface 0) */
                if(AltSettingNumber != ZeroBandwidthInterface)
                {
			    /* Read data from OUT endpoint and store in Isochronous_OUT_Buffer */
					USBFS_ReadOutEP(2, &OutBuffer[0], sizeof(OutBuffer));	
			        /* Re-arm the OUT Endpoint */
					USBFS_EnableOutEP(2);           
			        /*Rearm the IN Endpoint (EP1) */
			        USBFS_LoadInEP(EP1, &EP1_RAM[0],sizeof(EP1_RAM)); 
                }    
			}
		
			/* Check that ISO Interface is active (Not Interface 0) */
			if(AltSettingNumber != ZeroBandwidthInterface)
            {
				/* Check that all data has been transfered and IN Buffer is empty */
				if (USBFS_GetEPState(1) == USBFS_IN_BUFFER_EMPTY)
        		{
					if(InDataReady == 1) {
						USBFS_LoadInEP(EP1, USBFS_NULL, sizeof(EP1_RAM));
						InDataReady = 0;
					}
       			}
			}
		}
	}
    return (0);
}		

    

/* [] END OF FILE */

